package org.hashtree.jbrainhoney.dlap;

import java.util.EnumSet;

/** Enumerates BrainHoney DLAP API response codes. Includes response codes which are undocumented. */
public enum ResponseCode {
	ACCESSDENIED("The user does not hold privileges for the operation.", AccessDeniedException.class),
	BADREQUEST("Bad request.", BadRequestException.class),
	EXCEPTION("General exception.", GeneralException.class),
	HTTPSERVER("The request could not be handled by the HTTP server.", HttpServerException.class),
	ARGUMENT("The arguments in the query string or post data are incorrect.", InvalidArgumentException.class),
	INVALIDCREDENTIAL("The login credentials are invalid.", InvalidCredentialException.class),
	INVALIDOPERATION("The operation issued is invalid.", InvalidOperationException.class),
	NOAUTHENTICATION("The authentication token does not exist or has expired.", NoAuthenticationException.class),
	OK("Success.", null),
	SQL("The database experienced an issue handling the request.", SqlException.class);

	private final String meaning;
	private final Class<? extends ResponseCodeException> exceptionClass;

	private ResponseCode(final String meaning, final Class<? extends ResponseCodeException> exceptionClass) {
		this.meaning = meaning;
		this.exceptionClass = exceptionClass;
	}

	public String getMeaning() {
		return meaning;
	}

	public Class<? extends ResponseCodeException> getExceptionClass() {
		return exceptionClass;
	}

	public static ResponseCode valueOfExceptionClass(final Class<? extends ResponseCodeException> exceptionClass)
		throws IllegalArgumentException {

		if(exceptionClass == null) throw new IllegalArgumentException("Expected exceptionClass to not be null.");

		ResponseCode enumeration = null;

		for(ResponseCode element : EnumSet.allOf(ResponseCode.class)) {
			if(element.getExceptionClass() != null && element.getExceptionClass().equals(exceptionClass)) {
				enumeration = element;
				break;
			}
		}

		if(enumeration == null) throw new IllegalArgumentException("Expected exceptionClass match.");

		return enumeration;
	}

	public static ResponseCode valueOfMeaning(final String meaning) throws IllegalArgumentException {
		if(meaning == null) throw new IllegalArgumentException("Expected meaning to not be null.");

		ResponseCode enumeration = null;

		for(ResponseCode element : EnumSet.allOf(ResponseCode.class)) {
			if(element.getMeaning() != null && element.getMeaning().equals(meaning)) {
				enumeration = element;
				break;
			}
		}

		if(enumeration == null) throw new IllegalArgumentException("Expected meaning match.");

		return enumeration;
	}
}