package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which lists courses. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/GetCourseList">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class GetCourseListCommand implements Command {
	private final String domainId;
	private final String reference;
	private final String title;

	@SuppressWarnings("synthetic-access")
	private GetCourseListCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");

		domainId = builder.domainId;
		reference = builder.reference;
		title = builder.title;
	}

	public String getDomainId() {
		return domainId;
	}

	public String getReference() {
		return reference;
	}

	public String getTitle() {
		return title;
	}

	/** A builder of immutable GetCourseListCommand instances. */
	public final static class Builder implements Command.Builder {
		private String domainId;
		private String reference;
		private String title;

		public Builder() {
			super();
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setReference(final String reference) {
			this.reference = reference;

			return this;
		}

		public Builder setTitle(final String title) {
			this.title = title;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public GetCourseListCommand build() {
			return new GetCourseListCommand(this);
		}
	}
}