package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which gets the list of parent domains for a domain. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/GetDomainParentList">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class GetDomainParentListCommand implements Command {
	private final String domainId;

	@SuppressWarnings("synthetic-access")
	private GetDomainParentListCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.domainId == null) throw new IllegalArgumentException("Expected domainId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");

		domainId = builder.domainId;
	}

	public String getDomainId() {
		return domainId;
	}

	/** A builder of immutable GetDomainParentListCommand instances. */
	public final static class Builder implements Command.Builder {
		private final String domainId;

		public Builder(final String domainId) {
			super();

			this.domainId = domainId;
		}

		public Builder(final Integer domainId) {
			super();

			this.domainId = (domainId == null ? null : domainId.toString());
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public GetDomainParentListCommand build() {
			return new GetDomainParentListCommand(this);
		}
	}
}