package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which gets the entity type (Course, Domain, Group, or Section) for a given entityId. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/GetEntityType">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class GetEntityTypeCommand implements Command {
	private final String entityId;

	@SuppressWarnings("synthetic-access")
	private GetEntityTypeCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.entityId == null) throw new IllegalArgumentException("Expected entityId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.entityId))
			throw new IllegalArgumentException("Expected entityId to be valid.");

		entityId = builder.entityId;
	}

	public String getEntityId() {
		return entityId;
	}

	/** A builder of immutable GetEntityTypeCommand instances. */
	public final static class Builder implements Command.Builder {
		private final String entityId;

		public Builder(final String entityId) {
			super();

			this.entityId = entityId;
		}

		public Builder(final Integer entityId) {
			super();

			this.entityId = (entityId == null ? null : entityId.toString());
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public GetEntityTypeCommand build() {
			return new GetEntityTypeCommand(this);
		}
	}
}