package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which gets information for a section. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/GetSection">the BrainHoney DLAP API guide</a>} for this command.
 */
public final class GetSectionCommand implements Command {
	private final String sectionId;

	@SuppressWarnings("synthetic-access")
	private GetSectionCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.sectionId == null) throw new IllegalArgumentException("Expected sectionId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.sectionId))
			throw new IllegalArgumentException("Expected sectionId to be valid.");

		sectionId = builder.sectionId;
	}

	public String getSectionId() {
		return sectionId;
	}

	/** A builder of immutable GetSectionCommand instances. */
	public final static class Builder implements Command.Builder {
		private final String sectionId;

		public Builder(final String sectionId) {
			super();

			this.sectionId = sectionId;
		}

		public Builder(final Integer sectionId) {
			super();

			this.sectionId = (sectionId == null ? null : sectionId.toString());
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public GetSectionCommand build() {
			return new GetSectionCommand(this);
		}
	}
}