package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;
import org.hashtree.jbrainhoney.dlap.validate.UsernameAttributeValidator;

/**
 * A command which lists users. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/GetUserList">the BrainHoney DLAP API guide</a>} for this command.
 */
public final class GetUserListCommand implements Command {
	private final String domainId;
	private final String name;
	private final String username;

	@SuppressWarnings("synthetic-access")
	private GetUserListCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.username != null && !UsernameAttributeValidator.getInstance().isValid(builder.username))
			throw new IllegalArgumentException("Expected username to be valid.");

		domainId = builder.domainId;
		name = builder.name;
		username = builder.username;
	}

	public String getDomainId() {
		return domainId;
	}

	public String getName() {
		return name;
	}

	public String getUsername() {
		return username;
	}

	/** A builder of immutable GetUserListCommand instances. */
	public final static class Builder implements Command.Builder {
		private String domainId;
		private String name;
		private String username;

		public Builder() {
			super();
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setName(final String name) {
			this.name = name;

			return this;
		}

		public Builder setUsername(final String username) {
			this.username = username;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public GetUserListCommand build() {
			return new GetUserListCommand(this);
		}
	}
}