package org.hashtree.jbrainhoney.dlap.command;

import org.hashtree.jbrainhoney.dlap.element.Email;
import org.hashtree.jbrainhoney.dlap.validate.EntityIdAttributeValidator;

/**
 * A command which sends an e-mail to recipients that are enrolled in the same entity as that specified by the sender's
 * enrollmentId. See the {@link <a href="http://gls.agilix.com/Docs/Command/SendMail">the BrainHoney DLAP API guide</a>}
 * for this command.
 */
public final class SendEmailCommand implements Command {
	private final Email email;
	private final String enrollmentId;

	@SuppressWarnings("synthetic-access")
	private SendEmailCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.enrollmentId == null) throw new IllegalArgumentException("Expected enrollmentId to not be null.");
		else if(!EntityIdAttributeValidator.getInstance().isValid(builder.enrollmentId))
			throw new IllegalArgumentException("Expected enrollmentId to be valid.");

		email = builder.email;
		enrollmentId = builder.enrollmentId;
	}

	public Email getEmail() {
		return email;
	}

	public String getEnrollmentId() {
		return enrollmentId;
	}

	/** A builder of immutable SendEmailCommand instances. */
	public final static class Builder implements Command.Builder {
		private final Email email;
		private final String enrollmentId;

		public Builder(final String enrollmentId, final Email email) {
			super();

			this.enrollmentId = enrollmentId;
			this.email = email;
		}

		public Builder(final Integer enrollmentId, final Email email) {
			super();

			this.enrollmentId = (enrollmentId == null ? null : enrollmentId.toString());
			this.email = email;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public SendEmailCommand build() {
			return new SendEmailCommand(this);
		}
	}
}