package org.hashtree.jbrainhoney.dlap.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.hashtree.jbrainhoney.dlap.element.Domain;

/**
 * A command which updates the name and reference value of one or more domains. See the
 * {@link <a href="http://gls.agilix.com/Docs/Command/UpdateDomains">the BrainHoney DLAP API guide</a>} for this
 * command.
 */
public final class UpdateDomainsCommand implements Command {
	private final List<Domain> domains;

	@SuppressWarnings("synthetic-access")
	private UpdateDomainsCommand(final Builder builder) throws IllegalArgumentException {
		if(builder.domains == null || builder.domains.size() == 0)
			throw new IllegalArgumentException("Expected domains to not be null.");

		for(Domain domain : builder.domains) {
			if(domain == null) throw new IllegalArgumentException("Expected domain to not be null.");
			else if(domain.getDomainId() == null || (domain.getData() == null && domain.getName() == null
				&& domain.getReference() == null))

				throw new IllegalArgumentException("Expected domainId and data, name, or reference to not be null.");
		}

		domains = Collections.unmodifiableList(new ArrayList<Domain>(builder.domains));
	}

	public List<Domain> getDomains() {
		return domains;
	}

	/** A builder of immutable UpdateDomainsCommand instances. */
	public final static class Builder implements Command.Builder {
		private final List<Domain> domains;

		public Builder(final List<Domain> domains) {
			super();

			if(domains == null) this.domains = Collections.synchronizedList(new ArrayList<Domain>());
			else this.domains = Collections.synchronizedList(new ArrayList<Domain>(domains));
		}

		public Builder(final Domain domain) {
			super();

			if(domain == null) domains = Collections.synchronizedList(new ArrayList<Domain>());
			else domains = Collections.synchronizedList(new ArrayList<Domain>(Arrays.asList(domain)));
		}

		public Builder addDomain(final Domain domain) {
			domains.add(domain);

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public UpdateDomainsCommand build() {
			return new UpdateDomainsCommand(this);
		}
	}
}