package org.hashtree.jbrainhoney.dlap.element;

import java.util.Date;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import org.hashtree.jbrainhoney.compare.NullComparatorUtility;
import org.hashtree.jbrainhoney.dlap.enumeration.DateBoundary;
import org.hashtree.jbrainhoney.dlap.validate.*;
import org.hashtree.jbrainhoney.time.DateTimeUtility;
import org.joda.time.DateTime;

/** A user. */
public final class User implements XmlElement, Actor, Person, Comparable<User> {
	private static final long serialVersionUID = 936586491639520133L;
	private final Agent agent;
	private final DateTime creationDate;
	private final Element data;
	private final String domainId;
	private final String domainName;
	private final String email;
	private final String firstName;
	private final String guid;
	private final DateTime lastLoginDate;
	private final String lastName;
	private final String password;
	private final String passwordQuestion;
	private final String passwordAnswer;
	private final String reference;
	private final String sessionIndex;
	private final String userId;
	private final String username;
	private final String userspace;
	private final Integer version;

	@SuppressWarnings("synthetic-access")
	private User(final Builder builder) throws IllegalArgumentException {
		Element xmlElement = null;

		if(builder.data != null && builder.data.length() > 0) {
			nu.xom.Builder xmlBuilder = null;
			Document xmlDocument = null;

			try {
				xmlBuilder = new nu.xom.Builder();
				xmlDocument = xmlBuilder.build(builder.data, null);
			} catch(Exception exception) {
				throw new IllegalArgumentException("Expected data to be well formed XML.");
			}

			if(!xmlDocument.getRootElement().getLocalName().equals("data"))
				throw new IllegalArgumentException("Expected data root element.");

			xmlElement = (Element)xmlDocument.getRootElement().copy();
		}

		if(builder.domainId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.domainId))
			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(builder.email != null && !EmailAttributeValidator.getInstance().isValid(builder.email))
			throw new IllegalArgumentException("Expected email to be valid.");
		else if(builder.firstName != null && builder.firstName.length() > 256)
			throw new IllegalArgumentException("Expected firstName to be less than or equal to 256 characters.");
		else if(builder.guid != null && !GuidAttributeValidator.getInstance().isValid(builder.guid))
			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(builder.lastName != null && builder.lastName.length() > 256)
			throw new IllegalArgumentException("Expected lastName to be less than or equal to 256 characters.");
		else if(builder.password != null && builder.password.length() < 6)
			throw new IllegalArgumentException("Expected password to be at least 6 characters.");
		else if(builder.passwordQuestion != null && builder.passwordQuestion.length() > 256)
			throw new IllegalArgumentException("Expected passwordQuestion to be less than or equal to 256 characters.");
		else if(builder.passwordAnswer != null && builder.passwordAnswer.length() > 128)
			throw new IllegalArgumentException("Expected passwordAnswer to be less than or equal to 128 characters.");
		else if(builder.reference != null && !ReferenceAttributeValidator.getInstance().isValid(builder.reference))
			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(builder.sessionIndex != null && !GuidAttributeValidator.getInstance().isValid(builder.sessionIndex))
			throw new IllegalArgumentException("Expected sessionIndex to be valid.");
		else if(builder.userId != null && !EntityIdAttributeValidator.getInstance().isValid(builder.userId))
			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(builder.username != null && !UsernameAttributeValidator.getInstance().isValid(builder.username))
			throw new IllegalArgumentException("Expected username to be valid.");
		else if(builder.userspace != null && !UserspaceAttributeValidator.getInstance().isValid(builder.userspace))
			throw new IllegalArgumentException("Expected userspace to be valid.");
		else if(builder.version != null && !VersionAttributeValidator.getInstance().isValid(builder.version))
			throw new IllegalArgumentException("Expected version to be valid.");

		agent = builder.agent;
		creationDate = builder.creationDate;
		data = xmlElement;
		domainId = builder.domainId;
		domainName = builder.domainName;
		email = builder.email;
		firstName = builder.firstName;
		guid = builder.guid;
		lastLoginDate = builder.lastLoginDate;
		lastName = builder.lastName;
		password = builder.password;
		passwordQuestion = builder.passwordQuestion;
		passwordAnswer = builder.passwordAnswer;
		reference = builder.reference;
		sessionIndex = builder.sessionIndex;
		userId = builder.userId;
		username = builder.username;
		userspace = builder.userspace;
		version = builder.version;
	}

	public User(final nu.xom.Element element) throws IllegalArgumentException {
		if(!element.getQualifiedName().equals("user"))
			throw new IllegalArgumentException("Expected user XML element.");
		else if(element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("domainid").getValue()))

			throw new IllegalArgumentException("Expected domainId to be valid.");
		else if(element.getAttribute("email") != null && element.getAttribute("email").getValue().length() > 0
			&& !EmailAttributeValidator.getInstance().isValid(element.getAttribute("email").getValue()))

			throw new IllegalArgumentException("Expected email to be valid.");
		else if(element.getAttribute("firstname") != null && element.getAttribute("firstname").getValue().length() > 256)
			throw new IllegalArgumentException("Expected firstName to be less than or equal to 256 characters.");
		else if(element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0
			&& !GuidAttributeValidator.getInstance().isValid(element.getAttribute("guid").getValue()))

			throw new IllegalArgumentException("Expected guid to be valid.");
		else if(element.getAttribute("lastname") != null && element.getAttribute("lastname").getValue().length() > 256)
			throw new IllegalArgumentException("Expected lastName to be less than or equal to 256 characters.");
		else if(element.getAttribute("password") != null && element.getAttribute("password").getValue().length() < 6)
			throw new IllegalArgumentException("Expected password to be at least 6 characters.");
		else if(element.getAttribute("passwordquestion") != null && element.getAttribute("passwordquestion").getValue().length() > 256)
			throw new IllegalArgumentException("Expected passwordQuestion to be less than or equal to 256 characters.");
		else if(element.getAttribute("passwordanswer") != null && element.getAttribute("passwordanswer").getValue().length() > 128)
			throw new IllegalArgumentException("Expected passwordAnswer to be less than or equal to 128 characters.");
		else if(element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0
			&& !ReferenceAttributeValidator.getInstance().isValid(element.getAttribute("reference").getValue()))

			throw new IllegalArgumentException("Expected reference to be valid.");
		else if(element.getAttribute("sessionindex") != null && element.getAttribute("sessionindex").getValue().length() > 0
			&& !GuidAttributeValidator.getInstance().isValid(element.getAttribute("sessionindex").getValue()))

			throw new IllegalArgumentException("Expected sessionIndex to be valid.");
		else if(element.getAttribute("userid") != null && element.getAttribute("userid").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("userid").getValue()))

			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0
			&& !EntityIdAttributeValidator.getInstance().isValid(element.getAttribute("id").getValue()))

			throw new IllegalArgumentException("Expected userId to be valid.");
		else if(element.getAttribute("username") != null && element.getAttribute("username").getValue().length() > 0
			&& !UsernameAttributeValidator.getInstance().isValid(element.getAttribute("username").getValue()))

			throw new IllegalArgumentException("Expected username to be valid.");
		else if(element.getAttribute("userspace") != null && element.getAttribute("userspace").getValue().length() > 0
			&& !UserspaceAttributeValidator.getInstance().isValid(element.getAttribute("userspace").getValue()))

			throw new IllegalArgumentException("Expected userspace to be valid.");
		else if(element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0
			&& !VersionAttributeValidator.getInstance().isValid(element.getAttribute("version").getValue()))

			throw new IllegalArgumentException("Expected version to be valid.");

		agent = (element.getFirstChildElement("agent") != null ?
			new Agent((Element)element.getFirstChildElement("agent").copy()) : null);
		creationDate = (element.getAttribute("creationdate") != null && element.getAttribute("creationdate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("creationdate").getValue()) : null);
		data = (element.getFirstChildElement("data") != null ?
			(Element)element.getFirstChildElement("data").copy() : null);
		domainId = (element.getAttribute("domainid") != null && element.getAttribute("domainid").getValue().length() > 0 ?
			element.getAttribute("domainid").getValue() : null);
		domainName = (element.getAttribute("domainname") != null && element.getAttribute("domainname").getValue().length() > 0 ?
			element.getAttribute("domainname").getValue() : null);
		email = (element.getAttribute("email") != null && element.getAttribute("email").getValue().length() > 0 ?
			element.getAttribute("email").getValue() : null);
		firstName = (element.getAttribute("firstname") != null && element.getAttribute("firstname").getValue().length() > 0 ?
			element.getAttribute("firstname").getValue() : null);
		lastLoginDate = (element.getAttribute("lastlogindate") != null && element.getAttribute("lastlogindate").getValue().length() > 0 ?
			DateTimeUtility.valueOf(element.getAttribute("lastlogindate").getValue()) : null);
		lastName = (element.getAttribute("lastname") != null && element.getAttribute("lastname").getValue().length() > 0 ?
			element.getAttribute("lastname").getValue() : null);
		password = (element.getAttribute("password") != null && element.getAttribute("password").getValue().length() > 0 ?
			element.getAttribute("password").getValue() : null);
		passwordQuestion = (element.getAttribute("passwordquestion") != null && element.getAttribute("passwordquestion").getValue().length() > 0 ?
			element.getAttribute("passwordquestion").getValue() : null);
		passwordAnswer = (element.getAttribute("passwordanswer") != null && element.getAttribute("passwordanswer").getValue().length() > 0 ?
			element.getAttribute("passwordanswer").getValue() : null);
		reference = (element.getAttribute("reference") != null && element.getAttribute("reference").getValue().length() > 0 ?
			element.getAttribute("reference").getValue() : null);
		sessionIndex = (element.getAttribute("sessionIndex") != null && element.getAttribute("sessionIndex").getValue().length() > 0 ?
			element.getAttribute("sessionIndex").getValue() : null);

		// Referenced as userid and id.
		if(element.getAttribute("userid") != null && element.getAttribute("userid").getValue().length() > 0)
			userId = element.getAttribute("userid").getValue();
		else if(element.getAttribute("id") != null && element.getAttribute("id").getValue().length() > 0)
			userId = element.getAttribute("id").getValue();
		else userId = null;

		username = (element.getAttribute("username") != null && element.getAttribute("username").getValue().length() > 0 ?
			element.getAttribute("username").getValue() : null);
		userspace = (element.getAttribute("userspace") != null && element.getAttribute("userspace").getValue().length() > 0 ?
			element.getAttribute("userspace").getValue() : null);
		version = (element.getAttribute("version") != null && element.getAttribute("version").getValue().length() > 0 ?
			Integer.parseInt(element.getAttribute("version").getValue()) : null);

		// Referenced as guid and userguid.
		if(element.getAttribute("guid") != null && element.getAttribute("guid").getValue().length() > 0)
			guid = element.getAttribute("guid").getValue();
		else if(element.getAttribute("userguid") != null && element.getAttribute("userguid").getValue().length() > 0)
			guid = element.getAttribute("userguid").getValue();
		else guid = null;
	}

	@Override
	public int compareTo(final User user) throws NullPointerException {
		if(user == null) throw new NullPointerException("Expected user to not be null.");

		return NullComparatorUtility.compareIgnoreNullAndCase(this.userId, agent.getUserId());
	}

	public Agent getAgent() {
		return agent;
	}

	public DateTime getCreationDate() {
		return creationDate;
	}

	public Element getData() {
		return (data != null ? (Element)data.copy() : null);
	}

	public String getDomainId() {
		return domainId;
	}

	public String getDomainName() {
		return domainName;
	}

	public String getEmail() {
		return email;
	}

	public String getFirstName() {
		return firstName;
	}

	public String getGuid() {
		return guid;
	}

	public String getLastName() {
		return lastName;
	}

	public String getPassword() {
		return password;
	}

	public String getPasswordQuestion() {
		return passwordQuestion;
	}

	public String getPasswordAnswer() {
		return passwordAnswer;
	}

	public String getReference() {
		return reference;
	}

	public String getSessionIndex() {
		return sessionIndex;
	}

	public DateTime getLastLoginDate() {
		return lastLoginDate;
	}

	public String getUserId() {
		return userId;
	}

	public String getUsername() {
		return username;
	}

	public String getUserspace() {
		return userspace;
	}

	public Integer getVersion() {
		return version;
	}

	@Override
	public boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof User)) return false;
		else return toString().equals(((User)object).toString());
	}

	@Override
	public int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(User.class.getSimpleName() + "(");
		if(agent != null) stringBuilder.append("agent=" + agent.toString() + ", ");
		if(creationDate != null) stringBuilder.append("creationDate=" + creationDate.toString() + ", ");
		if(data != null) stringBuilder.append("data=" + data.toXML() + ", ");
		if(domainId != null) stringBuilder.append("domainId=" + domainId + ", ");
		if(domainName != null) stringBuilder.append("domainName=" + domainName + ", ");
		if(email != null) stringBuilder.append("email=" + email + ", ");
		if(firstName != null) stringBuilder.append("firstName=" + firstName + ", ");
		if(guid != null) stringBuilder.append("guid=" + guid + ", ");
		if(lastLoginDate != null) stringBuilder.append("lastLoginDate=" + lastLoginDate.toString() + ", ");
		if(lastName != null) stringBuilder.append("lastName=" + lastName + ", ");
		if(password != null) stringBuilder.append("password=" + password + ", ");
		if(passwordQuestion != null) stringBuilder.append("passwordQuestion=" + passwordQuestion + ", ");
		if(passwordAnswer != null) stringBuilder.append("passwordAnswer=" + passwordAnswer + ", ");
		if(reference != null) stringBuilder.append("reference=" + reference + ", ");
		if(sessionIndex != null) stringBuilder.append("sessionIndex=" + sessionIndex + ", ");
		if(userId != null) stringBuilder.append("userId=" + userId + ", ");
		if(username != null) stringBuilder.append("username=" + username + ", ");
		if(userspace != null) stringBuilder.append("userspace=" + userspace + ", ");
		if(version != null) stringBuilder.append("version=" + version.toString() + ", ");
		stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
		stringBuilder.append(")");

		return stringBuilder.toString();
	}

	@Override
	public Element toXmlElement() {
		Element element = new Element("user");

		if(agent != null) element.appendChild(agent.toXmlElement());
		if(creationDate != null) element.addAttribute(new Attribute("creationdate", creationDate.toString()));
		if(data != null) element.appendChild(data.copy());
		if(domainId != null) element.addAttribute(new Attribute("domainid", domainId));
		if(domainName != null) element.addAttribute(new Attribute("domainname", domainName));
		if(email != null) element.addAttribute(new Attribute("email", email));
		if(firstName != null) element.addAttribute(new Attribute("firstname", firstName));
		if(guid != null) element.addAttribute(new Attribute("guid", guid));
		if(lastLoginDate != null) element.addAttribute(new Attribute("lastlogindate", lastLoginDate.toString()));
		if(lastName != null) element.addAttribute(new Attribute("lastname", lastName));
		if(password != null) element.addAttribute(new Attribute("password", password));
		if(passwordQuestion != null) element.addAttribute(new Attribute("passwordquestion", passwordQuestion));
		if(passwordAnswer != null) element.addAttribute(new Attribute("passwordanswer", passwordAnswer));
		if(reference != null) element.addAttribute(new Attribute("reference", reference));
		if(sessionIndex != null) element.addAttribute(new Attribute("sessionindex", sessionIndex));
		if(userId != null) element.addAttribute(new Attribute("userid", userId));
		if(username != null) element.addAttribute(new Attribute("username", username));
		if(userspace != null) element.addAttribute(new Attribute("userspace", userspace));
		if(version != null) element.addAttribute(new Attribute("version", version.toString()));

		return element;
	}

	@Override
	public String toXmlString() {
		return toXmlElement().toXML();
	}

	/** A builder of immutable User elements. */
	public final static class Builder implements XmlElement.Builder {
		private Agent agent;
		private DateTime creationDate;
		private String data;
		private String domainId;
		private String domainName;
		private String email;
		private String firstName;
		private String guid;
		private DateTime lastLoginDate;
		private String lastName;
		private String password;
		private String passwordQuestion;
		private String passwordAnswer;
		private String reference;
		private String sessionIndex;
		private String userId;
		private String username;
		private String userspace;
		private Integer version;

		public Builder() {

		}

		public Builder setAgent(final Agent agent) {
			this.agent = agent;

			return this;
		}

		public Builder setCreationDate(final Date creationDate) {
			this.creationDate = (creationDate == null ? null : new DateTime(creationDate));

			return this;
		}

		public Builder setCreationDate(final DateTime creationDate) {
			this.creationDate = creationDate;

			return this;
		}

		public Builder setCreationDate(final DateBoundary creationDate) {
			this.creationDate = creationDate.getValue();

			return this;
		}

		public Builder setData(final String data) {
			this.data = data;

			return this;
		}

		public Builder setData(final Element data) {
			this.data = (data == null ? null : data.toXML());

			return this;
		}

		public Builder setDomainId(final String domainId) {
			this.domainId = domainId;

			return this;
		}

		public Builder setDomainId(final Integer domainId) {
			this.domainId = (domainId == null ? null : domainId.toString());

			return this;
		}

		public Builder setDomainName(final String domainName) {
			this.domainName = domainName;

			return this;
		}

		public Builder setEmail(final String email) {
			this.email = email;

			return this;
		}

		public Builder setFirstName(final String firstName) {
			this.firstName = firstName;

			return this;
		}

		public Builder setGuid(final String guid) {
			this.guid = guid;

			return this;
		}

		public Builder setLastLoginDate(final Date lastLoginDate) {
			this.lastLoginDate = (lastLoginDate == null ? null : new DateTime(lastLoginDate));

			return this;
		}

		public Builder setLastLoginDate(final DateTime lastLoginDate) {
			this.lastLoginDate = lastLoginDate;

			return this;
		}

		public Builder setLastLoginDate(final DateBoundary lastLoginDate) {
			this.lastLoginDate = lastLoginDate.getValue();

			return this;
		}

		public Builder setLastName(final String lastName) {
			this.lastName = lastName;

			return this;
		}

		public Builder setPassword(final String password) {
			this.password = password;

			return this;
		}

		public Builder setPasswordQuestion(final String passwordQuestion) {
			this.passwordQuestion = passwordQuestion;

			return this;
		}

		public Builder setPasswordAnswer(final String passwordAnswer) {
			this.passwordAnswer = passwordAnswer;

			return this;
		}

		public Builder setReference(final String reference) {
			this.reference = reference;

			return this;
		}

		public Builder setSessionIndex(final String sessionIndex) {
			this.sessionIndex = sessionIndex;

			return this;
		}

		public Builder setUserId(final String userId) {
			this.userId = userId;

			return this;
		}

		public Builder setUserId(final Integer userId) {
			this.userId = (userId == null ? null : userId.toString());

			return this;
		}

		public Builder setUsername(final String username) {
			this.username = username;

			return this;
		}

		public Builder setUserspace(final String userspace) {
			this.userspace = userspace;

			return this;
		}

		public Builder setVersion(final Integer version) {
			this.version = version;

			return this;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public User build() {
			return new User(this);
		}
	}
}