package org.hashtree.jbrainhoney.dlap.validate;

/** A validator class for an entityId attribute. */
public final class EntityIdAttributeValidator implements AttributeValidator<String> {
	private volatile static EntityIdAttributeValidator instance;

	private EntityIdAttributeValidator() {

	}

	@Override
	public boolean isValid(final String entityId) {
		if(entityId == null || entityId.length() == 0) return false;

		boolean valid = true;
		String[] ids = null;
		int index;

		if(entityId.startsWith(".")) index = entityId.indexOf("/", 1);
		else if(entityId.startsWith("//")) index = entityId.indexOf("/", 2);
		else if(entityId.startsWith("/")) index = entityId.indexOf("/", 1);
		else index = entityId.indexOf("/", 0);

		if(index == -1) {
			ids = new String[1];
			ids[0] = entityId;
		} else {
			ids = new String[2];
			ids[0] = entityId.substring(0, index);
			ids[1] = entityId.substring(index);
		}

		for(int i = 0; i < ids.length; i++) {
			String id = ids[i];
			boolean idValid = false;

			// <internal id> ::= string
			if(id.matches("^(\\d+)$")) idValid = true;
			// <guid> ::= {string}
			else if(id.matches("^(\\{{1}[0-9a-fA-F]{8}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{12}\\}{1})$"))
				idValid = true;
			// <reference> ::= /string
			else if(id.matches("^(/[^/\\|\\{\\}]+?)$")) idValid = true;
			// <userspace> ::= //string
			else if(id.matches("^(//[^/]+?)$")) idValid = true;
			// <current domain> ::= .
			else if(id.matches("^(\\.)$")) idValid = true;

			if(!idValid) {
				valid = false;
				break;
			}
		}

		return valid;
	}

	public boolean isValid(final Integer entityId) {
		return isValid(entityId.toString());
	}

	public static EntityIdAttributeValidator getInstance() {
		if(instance == null) synchronized(EntityIdAttributeValidator.class) {
			if(instance == null) instance = new EntityIdAttributeValidator();
		}

		return instance;
	}

	@Override
	public final java.lang.Object clone() throws CloneNotSupportedException {
		 throw new CloneNotSupportedException();
	}
}