package org.hashtree.jbrainhoney.network.http.request;

/** A string based implementation of a Body.*/
public final class StringBody implements Body {
	private static final long serialVersionUID = 66582650112749234L;
	private final String mimeType;
	private final String value;

	@SuppressWarnings("synthetic-access")
	private StringBody(final Builder builder) throws IllegalArgumentException {
		if(builder.mimeType == null) throw new IllegalArgumentException("Expected mimeType to not be null.");
		else if(builder.mimeType.trim().length() == 0)
			throw new IllegalArgumentException("Expected mimeType to have a length greater than 0.");
		else if(builder.value == null) throw new IllegalArgumentException("Expected value to not be null.");

		mimeType = builder.mimeType;
		value = builder.value;
	}

	@Override
	public final String getMimeType() {
		return mimeType;
	}

	@Override
	public final String getValue() {
		return value;
	}

	@Override
	public final boolean equals(final Object object) {
		if(object == this) return true;
		else if(!(object instanceof StringBody)) return false;
		else return toString().equals(((StringBody)object).toString());
	}

	@Override
	public final int hashCode() {
		int hashCode = 17;
		hashCode = (31 * hashCode) + (toString() != null ? toString().hashCode() : 0);

		return hashCode;
	}

	@Override
	public final String toString() {
		return StringBody.class.getSimpleName() + "(" + "mimeType=" + mimeType + (value != null ? ", value=" + value : "") + ")";
	}

	/** A builder of immutable StringBody instances. */
	public static class Builder implements Body.Builder {
		private final String mimeType;
		private final String value;

		public Builder(final String value, final String mimeType) {
			this.mimeType = mimeType;
			this.value = value;
		}

		@Override
		@SuppressWarnings("synthetic-access")
		public StringBody build() {
			return new StringBody(this);
		}
	}
}